////////////////////////////////////////////////////////////////////////////////
//                                  Include                                   //
////////////////////////////////////////////////////////////////////////////////


#include "Oracle.h"
#include "OPTtypes.h"
#include <fstream>
using namespace OPTtypes_di_unipi_it;
using namespace std;


////////////////////////////////////////////////////////////////////////////////
//                                   Main                                     //
////////////////////////////////////////////////////////////////////////////////


int main(int argc, char* argv[])
{
  //----- User: Settings ------------------------------------------------------1
  int iterations = 1000;
  //---------------------------------------------------------------------------1

  //----- Create --------------------------------------------------------------2

  // Create oracle object
  Oracle *oracle = new Oracle(argv[1]);

  // Variables
  int i, j, k;
  long uSize = oracle->getUSize();
  long xSize = 0;
  long xDims = oracle->getXDims();
  if (xDims == 1)
    xSize = oracle->getXDim1Size();
  else if (xDims == 2)
    xSize = oracle->getXDim1Size() * oracle->getXDim2Size();

  // Create uVector
  double* uVector = new double[uSize];

  // Create xVector
  double* xVector = new double[xSize];
  for (i = 0; i < xSize; i++)
    xVector[i] = 0;

  // Create iteration variables
  double fValue = 0;
  double* sgVector = new double[uSize];
  for (i = 0; i < uSize; i++)
    sgVector[i] = 0;
  double* xIterVector = new double[xSize];
  for (i = 0; i < xSize; i++)
    xIterVector[i] = 0;

  // Create step size
  double alfa = 0;
  double totalAlfa = 0;

  //---------------------------------------------------------------------------2

  //----- Solve ---------------------------------------------------------------3

  // Print Iterations
  cout << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << "--                             Iterations:                            --" << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << endl;

  // Initiate uVector
  for (i = 0; i < uSize; i++)
    uVector[i] = 0;

  // Iterate
  for (k = 0; k < iterations; k++) {

    // Oracle: (In: u vector) (Out: function value, subgradient vector, x vector)
    oracle->Solve(uVector, fValue, sgVector, xIterVector);

    //----- User: Step size --------------------------
    alfa = 1.0 / (1.0 + k);
    //------------------------------------------------

    // Update uVector
    for (i = 0; i < uSize; i++)
      uVector[i] -= alfa * sgVector[i];

    unsigned int* ucVector = oracle->getUCVector();
    i = 0;
    while (ucVector[i] < InINF) {
      if (uVector[ucVector[i]] < 0)
        uVector[ucVector[i]] = 0;
      i++;
    }

    //----- User: Update xVector ---------------------
    for (i = 0; i < xSize; i++)
      xVector[i] += xIterVector[i];
    //------------------------------------------------
  }

  // Print Solution
  cout << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << "--                              Solution:                             --" << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << endl;

  if (oracle->getIsMinProblem())
    cout << "  f = " << -fValue << endl;
  else
    cout << "  f = " << fValue << endl;

  cout << endl;

  ofstream ufile("uVector");
  for (i = 0; i < uSize; i++)
    ufile << "  u" << i + 1 << " = " << uVector[i] << endl;
  cout << "  u-vector written to file: uVector" << endl;

  cout << endl;

  //----- User: Create final xVector ---------------
  for (i = 0; i < xSize; i++)
    xVector[i] = xVector[i] / iterations;
  //------------------------------------------------

  ofstream xfile("xVector");
  if (xDims == 1) {
    for (i = 0; i < xSize; i++)
      xfile << "  x" << i + 1 << " = " << xVector[i] << endl;
  } else if (xDims == 2) {
    long xDim1Size = oracle->getXDim1Size();
    long xDim2Size = oracle->getXDim2Size();
    for (i = 0; i < xDim1Size; i++)
      for (j = 0; j < xDim2Size; j++)
	xfile << "  x" << i + 1 << "," << j + 1 << " = " << xVector[i * xDim1Size + j] << endl;
  }
  cout << "  x-vector written to file: xVector" << endl;

  cout << endl;
  cout << "------------------------------------------------------------------------" << endl;
  cout << endl;

  //---------------------------------------------------------------------------3

  //----- Delete --------------------------------------------------------------4

  // Delete vectors
  delete uVector;
  delete xVector;
  delete sgVector;
  delete xIterVector;

  // Delete oracle object
  delete oracle;

  //---------------------------------------------------------------------------4

  return(0);
}


////////////////////////////////////////////////////////////////////////////////
